# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from metronome import Metronome, AsyncMetronome
from tests.utils import assert_matches_type
from metronome._utils import parse_datetime
from metronome.types.v1 import (
    CreditGrantEditResponse,
    CreditGrantListResponse,
    CreditGrantVoidResponse,
    CreditGrantCreateResponse,
    CreditGrantListEntriesResponse,
)
from metronome.pagination import SyncCursorPage, AsyncCursorPage

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestCreditGrants:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_create(self, client: Metronome) -> None:
        credit_grant = client.v1.credit_grants.create(
            customer_id="9b85c1c1-5238-4f2a-a409-61412905e1e1",
            expires_at=parse_datetime("2022-04-01T00:00:00Z"),
            grant_amount={
                "amount": 1000,
                "credit_type_id": "5ae401dc-a648-4b49-9ac3-391bb5bc4d7b",
            },
            name="Acme Corp Promotional Credit Grant",
            paid_amount={
                "amount": 5000,
                "credit_type_id": "2714e483-4ff1-48e4-9e25-ac732e8f24f2",
            },
            priority=0.5,
        )
        assert_matches_type(CreditGrantCreateResponse, credit_grant, path=["response"])

    @parametrize
    def test_method_create_with_all_params(self, client: Metronome) -> None:
        credit_grant = client.v1.credit_grants.create(
            customer_id="9b85c1c1-5238-4f2a-a409-61412905e1e1",
            expires_at=parse_datetime("2022-04-01T00:00:00Z"),
            grant_amount={
                "amount": 1000,
                "credit_type_id": "5ae401dc-a648-4b49-9ac3-391bb5bc4d7b",
            },
            name="Acme Corp Promotional Credit Grant",
            paid_amount={
                "amount": 5000,
                "credit_type_id": "2714e483-4ff1-48e4-9e25-ac732e8f24f2",
            },
            priority=0.5,
            credit_grant_type="trial",
            custom_fields={"foo": "string"},
            effective_at=parse_datetime("2022-02-01T00:00:00Z"),
            invoice_date=parse_datetime("2019-12-27T18:11:19.117Z"),
            product_ids=["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
            reason="Incentivize new customer",
            rollover_settings={
                "expires_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                "priority": 0,
                "rollover_amount": {
                    "type": "MAX_PERCENTAGE",
                    "value": 0,
                },
            },
            uniqueness_key="x",
        )
        assert_matches_type(CreditGrantCreateResponse, credit_grant, path=["response"])

    @parametrize
    def test_raw_response_create(self, client: Metronome) -> None:
        response = client.v1.credit_grants.with_raw_response.create(
            customer_id="9b85c1c1-5238-4f2a-a409-61412905e1e1",
            expires_at=parse_datetime("2022-04-01T00:00:00Z"),
            grant_amount={
                "amount": 1000,
                "credit_type_id": "5ae401dc-a648-4b49-9ac3-391bb5bc4d7b",
            },
            name="Acme Corp Promotional Credit Grant",
            paid_amount={
                "amount": 5000,
                "credit_type_id": "2714e483-4ff1-48e4-9e25-ac732e8f24f2",
            },
            priority=0.5,
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        credit_grant = response.parse()
        assert_matches_type(CreditGrantCreateResponse, credit_grant, path=["response"])

    @parametrize
    def test_streaming_response_create(self, client: Metronome) -> None:
        with client.v1.credit_grants.with_streaming_response.create(
            customer_id="9b85c1c1-5238-4f2a-a409-61412905e1e1",
            expires_at=parse_datetime("2022-04-01T00:00:00Z"),
            grant_amount={
                "amount": 1000,
                "credit_type_id": "5ae401dc-a648-4b49-9ac3-391bb5bc4d7b",
            },
            name="Acme Corp Promotional Credit Grant",
            paid_amount={
                "amount": 5000,
                "credit_type_id": "2714e483-4ff1-48e4-9e25-ac732e8f24f2",
            },
            priority=0.5,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            credit_grant = response.parse()
            assert_matches_type(CreditGrantCreateResponse, credit_grant, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_list(self, client: Metronome) -> None:
        credit_grant = client.v1.credit_grants.list()
        assert_matches_type(SyncCursorPage[CreditGrantListResponse], credit_grant, path=["response"])

    @parametrize
    def test_method_list_with_all_params(self, client: Metronome) -> None:
        credit_grant = client.v1.credit_grants.list(
            limit=1,
            next_page="next_page",
            credit_grant_ids=["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
            credit_type_ids=["2714e483-4ff1-48e4-9e25-ac732e8f24f2"],
            customer_ids=["d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc", "0e5b8609-d901-4992-b394-c3c2e3f37b1c"],
            effective_before=parse_datetime("2022-02-01T00:00:00Z"),
            not_expiring_before=parse_datetime("2022-02-01T00:00:00Z"),
        )
        assert_matches_type(SyncCursorPage[CreditGrantListResponse], credit_grant, path=["response"])

    @parametrize
    def test_raw_response_list(self, client: Metronome) -> None:
        response = client.v1.credit_grants.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        credit_grant = response.parse()
        assert_matches_type(SyncCursorPage[CreditGrantListResponse], credit_grant, path=["response"])

    @parametrize
    def test_streaming_response_list(self, client: Metronome) -> None:
        with client.v1.credit_grants.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            credit_grant = response.parse()
            assert_matches_type(SyncCursorPage[CreditGrantListResponse], credit_grant, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_edit(self, client: Metronome) -> None:
        credit_grant = client.v1.credit_grants.edit(
            id="9b85c1c1-5238-4f2a-a409-61412905e1e1",
        )
        assert_matches_type(CreditGrantEditResponse, credit_grant, path=["response"])

    @parametrize
    def test_method_edit_with_all_params(self, client: Metronome) -> None:
        credit_grant = client.v1.credit_grants.edit(
            id="9b85c1c1-5238-4f2a-a409-61412905e1e1",
            credit_grant_type="credit_grant_type",
            expires_at=parse_datetime("2022-04-01T00:00:00Z"),
            name="Acme Corp Promotional Credit Grant",
        )
        assert_matches_type(CreditGrantEditResponse, credit_grant, path=["response"])

    @parametrize
    def test_raw_response_edit(self, client: Metronome) -> None:
        response = client.v1.credit_grants.with_raw_response.edit(
            id="9b85c1c1-5238-4f2a-a409-61412905e1e1",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        credit_grant = response.parse()
        assert_matches_type(CreditGrantEditResponse, credit_grant, path=["response"])

    @parametrize
    def test_streaming_response_edit(self, client: Metronome) -> None:
        with client.v1.credit_grants.with_streaming_response.edit(
            id="9b85c1c1-5238-4f2a-a409-61412905e1e1",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            credit_grant = response.parse()
            assert_matches_type(CreditGrantEditResponse, credit_grant, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_list_entries(self, client: Metronome) -> None:
        credit_grant = client.v1.credit_grants.list_entries()
        assert_matches_type(CreditGrantListEntriesResponse, credit_grant, path=["response"])

    @parametrize
    def test_method_list_entries_with_all_params(self, client: Metronome) -> None:
        credit_grant = client.v1.credit_grants.list_entries(
            next_page="next_page",
            sort="asc",
            credit_type_ids=["2714e483-4ff1-48e4-9e25-ac732e8f24f2"],
            customer_ids=["6a37bb88-8538-48c5-b37b-a41c836328bd"],
            ending_before=parse_datetime("2021-02-01T00:00:00Z"),
            starting_on=parse_datetime("2021-01-01T00:00:00Z"),
        )
        assert_matches_type(CreditGrantListEntriesResponse, credit_grant, path=["response"])

    @parametrize
    def test_raw_response_list_entries(self, client: Metronome) -> None:
        response = client.v1.credit_grants.with_raw_response.list_entries()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        credit_grant = response.parse()
        assert_matches_type(CreditGrantListEntriesResponse, credit_grant, path=["response"])

    @parametrize
    def test_streaming_response_list_entries(self, client: Metronome) -> None:
        with client.v1.credit_grants.with_streaming_response.list_entries() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            credit_grant = response.parse()
            assert_matches_type(CreditGrantListEntriesResponse, credit_grant, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_void(self, client: Metronome) -> None:
        credit_grant = client.v1.credit_grants.void(
            id="9b85c1c1-5238-4f2a-a409-61412905e1e1",
        )
        assert_matches_type(CreditGrantVoidResponse, credit_grant, path=["response"])

    @parametrize
    def test_method_void_with_all_params(self, client: Metronome) -> None:
        credit_grant = client.v1.credit_grants.void(
            id="9b85c1c1-5238-4f2a-a409-61412905e1e1",
            release_uniqueness_key=True,
            void_credit_purchase_invoice=True,
        )
        assert_matches_type(CreditGrantVoidResponse, credit_grant, path=["response"])

    @parametrize
    def test_raw_response_void(self, client: Metronome) -> None:
        response = client.v1.credit_grants.with_raw_response.void(
            id="9b85c1c1-5238-4f2a-a409-61412905e1e1",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        credit_grant = response.parse()
        assert_matches_type(CreditGrantVoidResponse, credit_grant, path=["response"])

    @parametrize
    def test_streaming_response_void(self, client: Metronome) -> None:
        with client.v1.credit_grants.with_streaming_response.void(
            id="9b85c1c1-5238-4f2a-a409-61412905e1e1",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            credit_grant = response.parse()
            assert_matches_type(CreditGrantVoidResponse, credit_grant, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncCreditGrants:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @parametrize
    async def test_method_create(self, async_client: AsyncMetronome) -> None:
        credit_grant = await async_client.v1.credit_grants.create(
            customer_id="9b85c1c1-5238-4f2a-a409-61412905e1e1",
            expires_at=parse_datetime("2022-04-01T00:00:00Z"),
            grant_amount={
                "amount": 1000,
                "credit_type_id": "5ae401dc-a648-4b49-9ac3-391bb5bc4d7b",
            },
            name="Acme Corp Promotional Credit Grant",
            paid_amount={
                "amount": 5000,
                "credit_type_id": "2714e483-4ff1-48e4-9e25-ac732e8f24f2",
            },
            priority=0.5,
        )
        assert_matches_type(CreditGrantCreateResponse, credit_grant, path=["response"])

    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncMetronome) -> None:
        credit_grant = await async_client.v1.credit_grants.create(
            customer_id="9b85c1c1-5238-4f2a-a409-61412905e1e1",
            expires_at=parse_datetime("2022-04-01T00:00:00Z"),
            grant_amount={
                "amount": 1000,
                "credit_type_id": "5ae401dc-a648-4b49-9ac3-391bb5bc4d7b",
            },
            name="Acme Corp Promotional Credit Grant",
            paid_amount={
                "amount": 5000,
                "credit_type_id": "2714e483-4ff1-48e4-9e25-ac732e8f24f2",
            },
            priority=0.5,
            credit_grant_type="trial",
            custom_fields={"foo": "string"},
            effective_at=parse_datetime("2022-02-01T00:00:00Z"),
            invoice_date=parse_datetime("2019-12-27T18:11:19.117Z"),
            product_ids=["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
            reason="Incentivize new customer",
            rollover_settings={
                "expires_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                "priority": 0,
                "rollover_amount": {
                    "type": "MAX_PERCENTAGE",
                    "value": 0,
                },
            },
            uniqueness_key="x",
        )
        assert_matches_type(CreditGrantCreateResponse, credit_grant, path=["response"])

    @parametrize
    async def test_raw_response_create(self, async_client: AsyncMetronome) -> None:
        response = await async_client.v1.credit_grants.with_raw_response.create(
            customer_id="9b85c1c1-5238-4f2a-a409-61412905e1e1",
            expires_at=parse_datetime("2022-04-01T00:00:00Z"),
            grant_amount={
                "amount": 1000,
                "credit_type_id": "5ae401dc-a648-4b49-9ac3-391bb5bc4d7b",
            },
            name="Acme Corp Promotional Credit Grant",
            paid_amount={
                "amount": 5000,
                "credit_type_id": "2714e483-4ff1-48e4-9e25-ac732e8f24f2",
            },
            priority=0.5,
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        credit_grant = await response.parse()
        assert_matches_type(CreditGrantCreateResponse, credit_grant, path=["response"])

    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncMetronome) -> None:
        async with async_client.v1.credit_grants.with_streaming_response.create(
            customer_id="9b85c1c1-5238-4f2a-a409-61412905e1e1",
            expires_at=parse_datetime("2022-04-01T00:00:00Z"),
            grant_amount={
                "amount": 1000,
                "credit_type_id": "5ae401dc-a648-4b49-9ac3-391bb5bc4d7b",
            },
            name="Acme Corp Promotional Credit Grant",
            paid_amount={
                "amount": 5000,
                "credit_type_id": "2714e483-4ff1-48e4-9e25-ac732e8f24f2",
            },
            priority=0.5,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            credit_grant = await response.parse()
            assert_matches_type(CreditGrantCreateResponse, credit_grant, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_list(self, async_client: AsyncMetronome) -> None:
        credit_grant = await async_client.v1.credit_grants.list()
        assert_matches_type(AsyncCursorPage[CreditGrantListResponse], credit_grant, path=["response"])

    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncMetronome) -> None:
        credit_grant = await async_client.v1.credit_grants.list(
            limit=1,
            next_page="next_page",
            credit_grant_ids=["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
            credit_type_ids=["2714e483-4ff1-48e4-9e25-ac732e8f24f2"],
            customer_ids=["d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc", "0e5b8609-d901-4992-b394-c3c2e3f37b1c"],
            effective_before=parse_datetime("2022-02-01T00:00:00Z"),
            not_expiring_before=parse_datetime("2022-02-01T00:00:00Z"),
        )
        assert_matches_type(AsyncCursorPage[CreditGrantListResponse], credit_grant, path=["response"])

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncMetronome) -> None:
        response = await async_client.v1.credit_grants.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        credit_grant = await response.parse()
        assert_matches_type(AsyncCursorPage[CreditGrantListResponse], credit_grant, path=["response"])

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncMetronome) -> None:
        async with async_client.v1.credit_grants.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            credit_grant = await response.parse()
            assert_matches_type(AsyncCursorPage[CreditGrantListResponse], credit_grant, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_edit(self, async_client: AsyncMetronome) -> None:
        credit_grant = await async_client.v1.credit_grants.edit(
            id="9b85c1c1-5238-4f2a-a409-61412905e1e1",
        )
        assert_matches_type(CreditGrantEditResponse, credit_grant, path=["response"])

    @parametrize
    async def test_method_edit_with_all_params(self, async_client: AsyncMetronome) -> None:
        credit_grant = await async_client.v1.credit_grants.edit(
            id="9b85c1c1-5238-4f2a-a409-61412905e1e1",
            credit_grant_type="credit_grant_type",
            expires_at=parse_datetime("2022-04-01T00:00:00Z"),
            name="Acme Corp Promotional Credit Grant",
        )
        assert_matches_type(CreditGrantEditResponse, credit_grant, path=["response"])

    @parametrize
    async def test_raw_response_edit(self, async_client: AsyncMetronome) -> None:
        response = await async_client.v1.credit_grants.with_raw_response.edit(
            id="9b85c1c1-5238-4f2a-a409-61412905e1e1",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        credit_grant = await response.parse()
        assert_matches_type(CreditGrantEditResponse, credit_grant, path=["response"])

    @parametrize
    async def test_streaming_response_edit(self, async_client: AsyncMetronome) -> None:
        async with async_client.v1.credit_grants.with_streaming_response.edit(
            id="9b85c1c1-5238-4f2a-a409-61412905e1e1",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            credit_grant = await response.parse()
            assert_matches_type(CreditGrantEditResponse, credit_grant, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_list_entries(self, async_client: AsyncMetronome) -> None:
        credit_grant = await async_client.v1.credit_grants.list_entries()
        assert_matches_type(CreditGrantListEntriesResponse, credit_grant, path=["response"])

    @parametrize
    async def test_method_list_entries_with_all_params(self, async_client: AsyncMetronome) -> None:
        credit_grant = await async_client.v1.credit_grants.list_entries(
            next_page="next_page",
            sort="asc",
            credit_type_ids=["2714e483-4ff1-48e4-9e25-ac732e8f24f2"],
            customer_ids=["6a37bb88-8538-48c5-b37b-a41c836328bd"],
            ending_before=parse_datetime("2021-02-01T00:00:00Z"),
            starting_on=parse_datetime("2021-01-01T00:00:00Z"),
        )
        assert_matches_type(CreditGrantListEntriesResponse, credit_grant, path=["response"])

    @parametrize
    async def test_raw_response_list_entries(self, async_client: AsyncMetronome) -> None:
        response = await async_client.v1.credit_grants.with_raw_response.list_entries()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        credit_grant = await response.parse()
        assert_matches_type(CreditGrantListEntriesResponse, credit_grant, path=["response"])

    @parametrize
    async def test_streaming_response_list_entries(self, async_client: AsyncMetronome) -> None:
        async with async_client.v1.credit_grants.with_streaming_response.list_entries() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            credit_grant = await response.parse()
            assert_matches_type(CreditGrantListEntriesResponse, credit_grant, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_void(self, async_client: AsyncMetronome) -> None:
        credit_grant = await async_client.v1.credit_grants.void(
            id="9b85c1c1-5238-4f2a-a409-61412905e1e1",
        )
        assert_matches_type(CreditGrantVoidResponse, credit_grant, path=["response"])

    @parametrize
    async def test_method_void_with_all_params(self, async_client: AsyncMetronome) -> None:
        credit_grant = await async_client.v1.credit_grants.void(
            id="9b85c1c1-5238-4f2a-a409-61412905e1e1",
            release_uniqueness_key=True,
            void_credit_purchase_invoice=True,
        )
        assert_matches_type(CreditGrantVoidResponse, credit_grant, path=["response"])

    @parametrize
    async def test_raw_response_void(self, async_client: AsyncMetronome) -> None:
        response = await async_client.v1.credit_grants.with_raw_response.void(
            id="9b85c1c1-5238-4f2a-a409-61412905e1e1",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        credit_grant = await response.parse()
        assert_matches_type(CreditGrantVoidResponse, credit_grant, path=["response"])

    @parametrize
    async def test_streaming_response_void(self, async_client: AsyncMetronome) -> None:
        async with async_client.v1.credit_grants.with_streaming_response.void(
            id="9b85c1c1-5238-4f2a-a409-61412905e1e1",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            credit_grant = await response.parse()
            assert_matches_type(CreditGrantVoidResponse, credit_grant, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, Optional

from ..._models import BaseModel

__all__ = ["ProService"]


class ProService(BaseModel):
    id: str

    max_amount: float
    """Maximum amount for the term."""

    product_id: str

    quantity: float
    """Quantity for the charge.

    Will be multiplied by unit_price to determine the amount.
    """

    unit_price: float
    """Unit price for the charge.

    Will be multiplied by quantity to determine the amount and must be specified.
    """

    custom_fields: Optional[Dict[str, str]] = None

    description: Optional[str] = None

    netsuite_sales_order_id: Optional[str] = None
    """This field's availability is dependent on your client's configuration."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from datetime import datetime

from .invoice import Invoice

__all__ = ["InvoiceListBreakdownsResponse"]


class InvoiceListBreakdownsResponse(Invoice):
    breakdown_end_timestamp: datetime

    breakdown_start_timestamp: datetime



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Union, Iterable
from datetime import datetime
from typing_extensions import Literal, Required, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["UsageListParams", "BillableMetric", "BillableMetricGroupBy"]


class UsageListParams(TypedDict, total=False):
    ending_before: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]

    starting_on: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]

    window_size: Required[Literal["HOUR", "DAY", "NONE"]]
    """
    A window_size of "day" or "hour" will return the usage for the specified period
    segmented into daily or hourly aggregates. A window_size of "none" will return a
    single usage aggregate for the entirety of the specified period.
    """

    next_page: str
    """Cursor that indicates where the next page of results should start."""

    billable_metrics: Iterable[BillableMetric]
    """A list of billable metrics to fetch usage for.

    If absent, all billable metrics will be returned.
    """

    customer_ids: List[str]
    """A list of Metronome customer IDs to fetch usage for.

    If absent, usage for all customers will be returned.
    """


class BillableMetricGroupBy(TypedDict, total=False):
    key: Required[str]
    """The name of the group_by key to use"""

    values: List[str]
    """Values of the group_by key to return in the query.

    If this field is omitted, all available values will be returned, up to a maximum
    of 200.
    """


class BillableMetric(TypedDict, total=False):
    id: Required[str]

    group_by: BillableMetricGroupBy



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Iterable
from datetime import datetime
from typing_extensions import Literal, Required, Annotated, TypedDict

from ...._utils import PropertyInfo

__all__ = ["PlanAddParams", "OverageRateAdjustment", "PriceAdjustment", "TrialSpec", "TrialSpecSpendingCap"]


class PlanAddParams(TypedDict, total=False):
    customer_id: Required[str]

    plan_id: Required[str]

    starting_on: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]
    """RFC 3339 timestamp for when the plan becomes active for this customer.

    Must be at 0:00 UTC (midnight).
    """

    ending_before: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """RFC 3339 timestamp for when the plan ends (exclusive) for this customer.

    Must be at 0:00 UTC (midnight).
    """

    net_payment_terms_days: float
    """Number of days after issuance of invoice after which the invoice is due (e.g.

    Net 30).
    """

    overage_rate_adjustments: Iterable[OverageRateAdjustment]
    """
    An optional list of overage rates that override the rates of the original plan
    configuration. These new rates will apply to all pricing ramps.
    """

    price_adjustments: Iterable[PriceAdjustment]
    """A list of price adjustments can be applied on top of the pricing in the plans.

    See the
    [price adjustments documentation](https://plans-docs.metronome.com/pricing/managing-plans/#price-adjustments)
    for details.
    """

    trial_spec: TrialSpec
    """A custom trial can be set for the customer's plan.

    See the
    [trial configuration documentation](https://docs.metronome.com/provisioning/configure-trials/)
    for details.
    """


class OverageRateAdjustment(TypedDict, total=False):
    custom_credit_type_id: Required[str]

    fiat_currency_credit_type_id: Required[str]

    to_fiat_conversion_factor: Required[float]
    """The overage cost in fiat currency for each credit of the custom credit type."""


class PriceAdjustment(TypedDict, total=False):
    adjustment_type: Required[Literal["percentage", "fixed", "override", "quantity"]]

    charge_id: Required[str]

    start_period: Required[float]
    """Used in price ramps.

    Indicates how many billing periods pass before the charge applies.
    """

    quantity: float
    """the overridden quantity for a fixed charge"""

    tier: float
    """Used in pricing tiers. Indicates at what metric value the price applies."""

    value: float
    """The amount of change to a price.

    Percentage and fixed adjustments can be positive or negative. Percentage-based
    adjustments should be decimals, e.g. -0.05 for a 5% discount.
    """


class TrialSpecSpendingCap(TypedDict, total=False):
    amount: Required[float]
    """The credit amount in the given denomination based on the credit type, e.g.

    US cents.
    """

    credit_type_id: Required[str]
    """The credit type ID for the spending cap."""


class TrialSpec(TypedDict, total=False):
    length_in_days: Required[float]
    """Length of the trial period in days."""

    spending_cap: TrialSpecSpendingCap



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Optional
from datetime import datetime

from ..._models import BaseModel

__all__ = ["UsageListResponse", "Data"]


class Data(BaseModel):
    billable_metric_id: str

    billable_metric_name: str

    customer_id: str

    end_timestamp: datetime

    start_timestamp: datetime

    value: Optional[float] = None

    groups: Optional[Dict[str, Optional[float]]] = None
    """Values will be either a number or null.

    Null indicates that there were no matches for the group_by value.
    """


class UsageListResponse(BaseModel):
    data: List[Data]

    next_page: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ...._models import BaseModel
from ...shared.id import ID

__all__ = ["RateCardArchiveResponse"]


class RateCardArchiveResponse(BaseModel):
    data: ID



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Union, Optional
from typing_extensions import TypeAlias

from ..._models import BaseModel
from ..shared.commit import Commit
from ..shared.credit import Credit

__all__ = ["ContractListBalancesResponse", "Data"]

Data: TypeAlias = Union[Commit, Credit]


class ContractListBalancesResponse(BaseModel):
    data: List[Data]

    next_page: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Union
from datetime import datetime
from typing_extensions import Literal, Required, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["UsageListWithGroupsParams", "GroupBy"]


class UsageListWithGroupsParams(TypedDict, total=False):
    billable_metric_id: Required[str]

    customer_id: Required[str]

    window_size: Required[Literal["HOUR", "DAY", "NONE"]]
    """
    A window_size of "day" or "hour" will return the usage for the specified period
    segmented into daily or hourly aggregates. A window_size of "none" will return a
    single usage aggregate for the entirety of the specified period.
    """

    limit: int
    """Max number of results that should be returned"""

    next_page: str
    """Cursor that indicates where the next page of results should start."""

    current_period: bool
    """If true, will return the usage for the current billing period.

    Will return an error if the customer is currently uncontracted or starting_on
    and ending_before are specified when this is true.
    """

    ending_before: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]

    group_by: GroupBy

    starting_on: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]


class GroupBy(TypedDict, total=False):
    key: Required[str]
    """The name of the group_by key to use"""

    values: List[str]
    """Values of the group_by key to return in the query.

    Omit this if you'd like all values for the key returned.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, List, Union, Iterable
from datetime import datetime
from typing_extensions import Literal

import httpx

from ...._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ...._utils import maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._base_client import make_request_options
from ....types.v1.customers import commit_list_params, commit_create_params, commit_update_end_date_params
from ....types.v1.customers.commit_list_response import CommitListResponse
from ....types.v1.customers.commit_create_response import CommitCreateResponse
from ....types.v1.customers.commit_update_end_date_response import CommitUpdateEndDateResponse

__all__ = ["CommitsResource", "AsyncCommitsResource"]


class CommitsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> CommitsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Metronome-Industries/metronome-python#accessing-raw-response-data-eg-headers
        """
        return CommitsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> CommitsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Metronome-Industries/metronome-python#with_streaming_response
        """
        return CommitsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        access_schedule: commit_create_params.AccessSchedule,
        customer_id: str,
        priority: float,
        product_id: str,
        type: Literal["PREPAID", "POSTPAID"],
        applicable_contract_ids: List[str] | NotGiven = NOT_GIVEN,
        applicable_product_ids: List[str] | NotGiven = NOT_GIVEN,
        applicable_product_tags: List[str] | NotGiven = NOT_GIVEN,
        custom_fields: Dict[str, str] | NotGiven = NOT_GIVEN,
        description: str | NotGiven = NOT_GIVEN,
        invoice_contract_id: str | NotGiven = NOT_GIVEN,
        invoice_schedule: commit_create_params.InvoiceSchedule | NotGiven = NOT_GIVEN,
        name: str | NotGiven = NOT_GIVEN,
        netsuite_sales_order_id: str | NotGiven = NOT_GIVEN,
        rate_type: Literal["COMMIT_RATE", "LIST_RATE"] | NotGiven = NOT_GIVEN,
        salesforce_opportunity_id: str | NotGiven = NOT_GIVEN,
        specifiers: Iterable[commit_create_params.Specifier] | NotGiven = NOT_GIVEN,
        uniqueness_key: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CommitCreateResponse:
        """
        Create a new commit at the customer level.

        Args:
          access_schedule: Schedule for distributing the commit to the customer. For "POSTPAID" commits
              only one schedule item is allowed and amount must match invoice_schedule total.

          priority: If multiple credits or commits are applicable, the one with the lower priority
              will apply first.

          product_id: ID of the fixed product associated with the commit. This is required because
              products are used to invoice the commit amount.

          applicable_contract_ids: Which contract the commit applies to. If not provided, the commit applies to all
              contracts.

          applicable_product_ids: Which products the commit applies to. If applicable_product_ids,
              applicable_product_tags or specifiers are not provided, the commit applies to
              all products.

          applicable_product_tags: Which tags the commit applies to. If applicable_product_ids,
              applicable_product_tags or specifiers are not provided, the commit applies to
              all products.

          description: Used only in UI/API. It is not exposed to end customers.

          invoice_contract_id: The contract that this commit will be billed on. This is required for "POSTPAID"
              commits and for "PREPAID" commits unless there is no invoice schedule above
              (i.e., the commit is 'free').

          invoice_schedule: Required for "POSTPAID" commits: the true up invoice will be generated at this
              time and only one schedule item is allowed; the total must match
              accesss_schedule amount. Optional for "PREPAID" commits: if not provided, this
              will be a "complimentary" commit with no invoice.

          name: displayed on invoices

          netsuite_sales_order_id: This field's availability is dependent on your client's configuration.

          salesforce_opportunity_id: This field's availability is dependent on your client's configuration.

          specifiers: List of filters that determine what kind of customer usage draws down a commit
              or credit. A customer's usage needs to meet the condition of at least one of the
              specifiers to contribute to a commit's or credit's drawdown. This field cannot
              be used together with `applicable_product_ids` or `applicable_product_tags`.

          uniqueness_key: Prevents the creation of duplicates. If a request to create a commit or credit
              is made with a uniqueness key that was previously used to create a commit or
              credit, a new record will not be created and the request will fail with a 409
              error.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v1/contracts/customerCommits/create",
            body=maybe_transform(
                {
                    "access_schedule": access_schedule,
                    "customer_id": customer_id,
                    "priority": priority,
                    "product_id": product_id,
                    "type": type,
                    "applicable_contract_ids": applicable_contract_ids,
                    "applicable_product_ids": applicable_product_ids,
                    "applicable_product_tags": applicable_product_tags,
                    "custom_fields": custom_fields,
                    "description": description,
                    "invoice_contract_id": invoice_contract_id,
                    "invoice_schedule": invoice_schedule,
                    "name": name,
                    "netsuite_sales_order_id": netsuite_sales_order_id,
                    "rate_type": rate_type,
                    "salesforce_opportunity_id": salesforce_opportunity_id,
                    "specifiers": specifiers,
                    "uniqueness_key": uniqueness_key,
                },
                commit_create_params.CommitCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CommitCreateResponse,
        )

    def list(
        self,
        *,
        customer_id: str,
        commit_id: str | NotGiven = NOT_GIVEN,
        covering_date: Union[str, datetime] | NotGiven = NOT_GIVEN,
        effective_before: Union[str, datetime] | NotGiven = NOT_GIVEN,
        include_archived: bool | NotGiven = NOT_GIVEN,
        include_balance: bool | NotGiven = NOT_GIVEN,
        include_contract_commits: bool | NotGiven = NOT_GIVEN,
        include_ledgers: bool | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        next_page: str | NotGiven = NOT_GIVEN,
        starting_at: Union[str, datetime] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CommitListResponse:
        """
        List commits.

        Args:
          covering_date: Include only commits that have access schedules that "cover" the provided date

          effective_before: Include only commits that have any access before the provided date (exclusive)

          include_archived: Include archived commits and commits from archived contracts.

          include_balance: Include the balance in the response. Setting this flag may cause the query to be
              slower.

          include_contract_commits: Include commits on the contract level.

          include_ledgers: Include commit ledgers in the response. Setting this flag may cause the query to
              be slower.

          limit: The maximum number of commits to return. Defaults to 25.

          next_page: The next page token from a previous response.

          starting_at: Include only commits that have any access on or after the provided date

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v1/contracts/customerCommits/list",
            body=maybe_transform(
                {
                    "customer_id": customer_id,
                    "commit_id": commit_id,
                    "covering_date": covering_date,
                    "effective_before": effective_before,
                    "include_archived": include_archived,
                    "include_balance": include_balance,
                    "include_contract_commits": include_contract_commits,
                    "include_ledgers": include_ledgers,
                    "limit": limit,
                    "next_page": next_page,
                    "starting_at": starting_at,
                },
                commit_list_params.CommitListParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CommitListResponse,
        )

    def update_end_date(
        self,
        *,
        commit_id: str,
        customer_id: str,
        access_ending_before: Union[str, datetime] | NotGiven = NOT_GIVEN,
        invoices_ending_before: Union[str, datetime] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CommitUpdateEndDateResponse:
        """Pull forward the end date of a prepaid commit.

        Use the "edit a commit" endpoint
        to extend the end date of a prepaid commit, or to make other edits to the
        commit.

        Args:
          commit_id: ID of the commit to update. Only supports "PREPAID" commits.

          customer_id: ID of the customer whose commit is to be updated

          access_ending_before: RFC 3339 timestamp indicating when access to the commit will end and it will no
              longer be possible to draw it down (exclusive). If not provided, the access will
              not be updated.

          invoices_ending_before: RFC 3339 timestamp indicating when the commit will stop being invoiced
              (exclusive). If not provided, the invoice schedule will not be updated.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v1/contracts/customerCommits/updateEndDate",
            body=maybe_transform(
                {
                    "commit_id": commit_id,
                    "customer_id": customer_id,
                    "access_ending_before": access_ending_before,
                    "invoices_ending_before": invoices_ending_before,
                },
                commit_update_end_date_params.CommitUpdateEndDateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CommitUpdateEndDateResponse,
        )


class AsyncCommitsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncCommitsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Metronome-Industries/metronome-python#accessing-raw-response-data-eg-headers
        """
        return AsyncCommitsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncCommitsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Metronome-Industries/metronome-python#with_streaming_response
        """
        return AsyncCommitsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        access_schedule: commit_create_params.AccessSchedule,
        customer_id: str,
        priority: float,
        product_id: str,
        type: Literal["PREPAID", "POSTPAID"],
        applicable_contract_ids: List[str] | NotGiven = NOT_GIVEN,
        applicable_product_ids: List[str] | NotGiven = NOT_GIVEN,
        applicable_product_tags: List[str] | NotGiven = NOT_GIVEN,
        custom_fields: Dict[str, str] | NotGiven = NOT_GIVEN,
        description: str | NotGiven = NOT_GIVEN,
        invoice_contract_id: str | NotGiven = NOT_GIVEN,
        invoice_schedule: commit_create_params.InvoiceSchedule | NotGiven = NOT_GIVEN,
        name: str | NotGiven = NOT_GIVEN,
        netsuite_sales_order_id: str | NotGiven = NOT_GIVEN,
        rate_type: Literal["COMMIT_RATE", "LIST_RATE"] | NotGiven = NOT_GIVEN,
        salesforce_opportunity_id: str | NotGiven = NOT_GIVEN,
        specifiers: Iterable[commit_create_params.Specifier] | NotGiven = NOT_GIVEN,
        uniqueness_key: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CommitCreateResponse:
        """
        Create a new commit at the customer level.

        Args:
          access_schedule: Schedule for distributing the commit to the customer. For "POSTPAID" commits
              only one schedule item is allowed and amount must match invoice_schedule total.

          priority: If multiple credits or commits are applicable, the one with the lower priority
              will apply first.

          product_id: ID of the fixed product associated with the commit. This is required because
              products are used to invoice the commit amount.

          applicable_contract_ids: Which contract the commit applies to. If not provided, the commit applies to all
              contracts.

          applicable_product_ids: Which products the commit applies to. If applicable_product_ids,
              applicable_product_tags or specifiers are not provided, the commit applies to
              all products.

          applicable_product_tags: Which tags the commit applies to. If applicable_product_ids,
              applicable_product_tags or specifiers are not provided, the commit applies to
              all products.

          description: Used only in UI/API. It is not exposed to end customers.

          invoice_contract_id: The contract that this commit will be billed on. This is required for "POSTPAID"
              commits and for "PREPAID" commits unless there is no invoice schedule above
              (i.e., the commit is 'free').

          invoice_schedule: Required for "POSTPAID" commits: the true up invoice will be generated at this
              time and only one schedule item is allowed; the total must match
              accesss_schedule amount. Optional for "PREPAID" commits: if not provided, this
              will be a "complimentary" commit with no invoice.

          name: displayed on invoices

          netsuite_sales_order_id: This field's availability is dependent on your client's configuration.

          salesforce_opportunity_id: This field's availability is dependent on your client's configuration.

          specifiers: List of filters that determine what kind of customer usage draws down a commit
              or credit. A customer's usage needs to meet the condition of at least one of the
              specifiers to contribute to a commit's or credit's drawdown. This field cannot
              be used together with `applicable_product_ids` or `applicable_product_tags`.

          uniqueness_key: Prevents the creation of duplicates. If a request to create a commit or credit
              is made with a uniqueness key that was previously used to create a commit or
              credit, a new record will not be created and the request will fail with a 409
              error.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v1/contracts/customerCommits/create",
            body=await async_maybe_transform(
                {
                    "access_schedule": access_schedule,
                    "customer_id": customer_id,
                    "priority": priority,
                    "product_id": product_id,
                    "type": type,
                    "applicable_contract_ids": applicable_contract_ids,
                    "applicable_product_ids": applicable_product_ids,
                    "applicable_product_tags": applicable_product_tags,
                    "custom_fields": custom_fields,
                    "description": description,
                    "invoice_contract_id": invoice_contract_id,
                    "invoice_schedule": invoice_schedule,
                    "name": name,
                    "netsuite_sales_order_id": netsuite_sales_order_id,
                    "rate_type": rate_type,
                    "salesforce_opportunity_id": salesforce_opportunity_id,
                    "specifiers": specifiers,
                    "uniqueness_key": uniqueness_key,
                },
                commit_create_params.CommitCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CommitCreateResponse,
        )

    async def list(
        self,
        *,
        customer_id: str,
        commit_id: str | NotGiven = NOT_GIVEN,
        covering_date: Union[str, datetime] | NotGiven = NOT_GIVEN,
        effective_before: Union[str, datetime] | NotGiven = NOT_GIVEN,
        include_archived: bool | NotGiven = NOT_GIVEN,
        include_balance: bool | NotGiven = NOT_GIVEN,
        include_contract_commits: bool | NotGiven = NOT_GIVEN,
        include_ledgers: bool | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        next_page: str | NotGiven = NOT_GIVEN,
        starting_at: Union[str, datetime] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CommitListResponse:
        """
        List commits.

        Args:
          covering_date: Include only commits that have access schedules that "cover" the provided date

          effective_before: Include only commits that have any access before the provided date (exclusive)

          include_archived: Include archived commits and commits from archived contracts.

          include_balance: Include the balance in the response. Setting this flag may cause the query to be
              slower.

          include_contract_commits: Include commits on the contract level.

          include_ledgers: Include commit ledgers in the response. Setting this flag may cause the query to
              be slower.

          limit: The maximum number of commits to return. Defaults to 25.

          next_page: The next page token from a previous response.

          starting_at: Include only commits that have any access on or after the provided date

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v1/contracts/customerCommits/list",
            body=await async_maybe_transform(
                {
                    "customer_id": customer_id,
                    "commit_id": commit_id,
                    "covering_date": covering_date,
                    "effective_before": effective_before,
                    "include_archived": include_archived,
                    "include_balance": include_balance,
                    "include_contract_commits": include_contract_commits,
                    "include_ledgers": include_ledgers,
                    "limit": limit,
                    "next_page": next_page,
                    "starting_at": starting_at,
                },
                commit_list_params.CommitListParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CommitListResponse,
        )

    async def update_end_date(
        self,
        *,
        commit_id: str,
        customer_id: str,
        access_ending_before: Union[str, datetime] | NotGiven = NOT_GIVEN,
        invoices_ending_before: Union[str, datetime] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CommitUpdateEndDateResponse:
        """Pull forward the end date of a prepaid commit.

        Use the "edit a commit" endpoint
        to extend the end date of a prepaid commit, or to make other edits to the
        commit.

        Args:
          commit_id: ID of the commit to update. Only supports "PREPAID" commits.

          customer_id: ID of the customer whose commit is to be updated

          access_ending_before: RFC 3339 timestamp indicating when access to the commit will end and it will no
              longer be possible to draw it down (exclusive). If not provided, the access will
              not be updated.

          invoices_ending_before: RFC 3339 timestamp indicating when the commit will stop being invoiced
              (exclusive). If not provided, the invoice schedule will not be updated.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v1/contracts/customerCommits/updateEndDate",
            body=await async_maybe_transform(
                {
                    "commit_id": commit_id,
                    "customer_id": customer_id,
                    "access_ending_before": access_ending_before,
                    "invoices_ending_before": invoices_ending_before,
                },
                commit_update_end_date_params.CommitUpdateEndDateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CommitUpdateEndDateResponse,
        )


class CommitsResourceWithRawResponse:
    def __init__(self, commits: CommitsResource) -> None:
        self._commits = commits

        self.create = to_raw_response_wrapper(
            commits.create,
        )
        self.list = to_raw_response_wrapper(
            commits.list,
        )
        self.update_end_date = to_raw_response_wrapper(
            commits.update_end_date,
        )


class AsyncCommitsResourceWithRawResponse:
    def __init__(self, commits: AsyncCommitsResource) -> None:
        self._commits = commits

        self.create = async_to_raw_response_wrapper(
            commits.create,
        )
        self.list = async_to_raw_response_wrapper(
            commits.list,
        )
        self.update_end_date = async_to_raw_response_wrapper(
            commits.update_end_date,
        )


class CommitsResourceWithStreamingResponse:
    def __init__(self, commits: CommitsResource) -> None:
        self._commits = commits

        self.create = to_streamed_response_wrapper(
            commits.create,
        )
        self.list = to_streamed_response_wrapper(
            commits.list,
        )
        self.update_end_date = to_streamed_response_wrapper(
            commits.update_end_date,
        )


class AsyncCommitsResourceWithStreamingResponse:
    def __init__(self, commits: AsyncCommitsResource) -> None:
        self._commits = commits

        self.create = async_to_streamed_response_wrapper(
            commits.create,
        )
        self.list = async_to_streamed_response_wrapper(
            commits.list,
        )
        self.update_end_date = async_to_streamed_response_wrapper(
            commits.update_end_date,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["ContractArchiveParams"]


class ContractArchiveParams(TypedDict, total=False):
    contract_id: Required[str]
    """ID of the contract to archive"""

    customer_id: Required[str]
    """ID of the customer whose contract is to be archived"""

    void_invoices: Required[bool]
    """
    If false, the existing finalized invoices will remain after the contract is
    archived.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, List, Union, Iterable
from datetime import datetime
from typing_extensions import Literal, Required, Annotated, TypeAlias, TypedDict

from ..._utils import PropertyInfo
from ..shared_params.tier import Tier

__all__ = [
    "ContractAmendParams",
    "Commit",
    "CommitAccessSchedule",
    "CommitAccessScheduleScheduleItem",
    "CommitHierarchyConfiguration",
    "CommitHierarchyConfigurationChildAccess",
    "CommitHierarchyConfigurationChildAccessCommitHierarchyChildAccessAll",
    "CommitHierarchyConfigurationChildAccessCommitHierarchyChildAccessNone",
    "CommitHierarchyConfigurationChildAccessCommitHierarchyChildAccessContractIDs",
    "CommitInvoiceSchedule",
    "CommitInvoiceScheduleRecurringSchedule",
    "CommitInvoiceScheduleScheduleItem",
    "CommitPaymentGateConfig",
    "CommitPaymentGateConfigPrecalculatedTaxConfig",
    "CommitPaymentGateConfigStripeConfig",
    "CommitSpecifier",
    "Credit",
    "CreditAccessSchedule",
    "CreditAccessScheduleScheduleItem",
    "CreditHierarchyConfiguration",
    "CreditHierarchyConfigurationChildAccess",
    "CreditHierarchyConfigurationChildAccessCommitHierarchyChildAccessAll",
    "CreditHierarchyConfigurationChildAccessCommitHierarchyChildAccessNone",
    "CreditHierarchyConfigurationChildAccessCommitHierarchyChildAccessContractIDs",
    "CreditSpecifier",
    "Discount",
    "DiscountSchedule",
    "DiscountScheduleRecurringSchedule",
    "DiscountScheduleScheduleItem",
    "Override",
    "OverrideOverrideSpecifier",
    "OverrideOverwriteRate",
    "OverrideTier",
    "ProfessionalService",
    "ResellerRoyalty",
    "ResellerRoyaltyAwsOptions",
    "ResellerRoyaltyGcpOptions",
    "ScheduledCharge",
    "ScheduledChargeSchedule",
    "ScheduledChargeScheduleRecurringSchedule",
    "ScheduledChargeScheduleScheduleItem",
]


class ContractAmendParams(TypedDict, total=False):
    contract_id: Required[str]
    """ID of the contract to amend"""

    customer_id: Required[str]
    """ID of the customer whose contract is to be amended"""

    starting_at: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]
    """inclusive start time for the amendment"""

    commits: Iterable[Commit]

    credits: Iterable[Credit]

    custom_fields: Dict[str, str]

    discounts: Iterable[Discount]
    """This field's availability is dependent on your client's configuration."""

    netsuite_sales_order_id: str
    """This field's availability is dependent on your client's configuration."""

    overrides: Iterable[Override]

    professional_services: Iterable[ProfessionalService]
    """This field's availability is dependent on your client's configuration."""

    reseller_royalties: Iterable[ResellerRoyalty]
    """This field's availability is dependent on your client's configuration."""

    salesforce_opportunity_id: str
    """This field's availability is dependent on your client's configuration."""

    scheduled_charges: Iterable[ScheduledCharge]

    total_contract_value: float
    """This field's availability is dependent on your client's configuration."""


class CommitAccessScheduleScheduleItem(TypedDict, total=False):
    amount: Required[float]

    ending_before: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]
    """RFC 3339 timestamp (exclusive)"""

    starting_at: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]
    """RFC 3339 timestamp (inclusive)"""


class CommitAccessSchedule(TypedDict, total=False):
    schedule_items: Required[Iterable[CommitAccessScheduleScheduleItem]]

    credit_type_id: str
    """Defaults to USD (cents) if not passed"""


class CommitHierarchyConfigurationChildAccessCommitHierarchyChildAccessAll(TypedDict, total=False):
    type: Required[Literal["ALL"]]


class CommitHierarchyConfigurationChildAccessCommitHierarchyChildAccessNone(TypedDict, total=False):
    type: Required[Literal["NONE"]]


class CommitHierarchyConfigurationChildAccessCommitHierarchyChildAccessContractIDs(TypedDict, total=False):
    contract_ids: Required[List[str]]

    type: Required[Literal["CONTRACT_IDS"]]


CommitHierarchyConfigurationChildAccess: TypeAlias = Union[
    CommitHierarchyConfigurationChildAccessCommitHierarchyChildAccessAll,
    CommitHierarchyConfigurationChildAccessCommitHierarchyChildAccessNone,
    CommitHierarchyConfigurationChildAccessCommitHierarchyChildAccessContractIDs,
]


class CommitHierarchyConfiguration(TypedDict, total=False):
    child_access: Required[CommitHierarchyConfigurationChildAccess]


class CommitInvoiceScheduleRecurringSchedule(TypedDict, total=False):
    amount_distribution: Required[Literal["DIVIDED", "DIVIDED_ROUNDED", "EACH"]]

    ending_before: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]
    """RFC 3339 timestamp (exclusive)."""

    frequency: Required[Literal["MONTHLY", "QUARTERLY", "SEMI_ANNUAL", "ANNUAL"]]

    starting_at: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]
    """RFC 3339 timestamp (inclusive)."""

    amount: float
    """Amount for the charge.

    Can be provided instead of unit_price and quantity. If amount is sent, the
    unit_price is assumed to be the amount and quantity is inferred to be 1.
    """

    quantity: float
    """Quantity for the charge.

    Will be multiplied by unit_price to determine the amount and must be specified
    with unit_price. If specified amount cannot be provided.
    """

    unit_price: float
    """Unit price for the charge.

    Will be multiplied by quantity to determine the amount and must be specified
    with quantity. If specified amount cannot be provided.
    """


class CommitInvoiceScheduleScheduleItem(TypedDict, total=False):
    timestamp: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]
    """timestamp of the scheduled event"""

    amount: float
    """Amount for the charge.

    Can be provided instead of unit_price and quantity. If amount is sent, the
    unit_price is assumed to be the amount and quantity is inferred to be 1.
    """

    quantity: float
    """Quantity for the charge.

    Will be multiplied by unit_price to determine the amount and must be specified
    with unit_price. If specified amount cannot be provided.
    """

    unit_price: float
    """Unit price for the charge.

    Will be multiplied by quantity to determine the amount and must be specified
    with quantity. If specified amount cannot be provided.
    """


class CommitInvoiceSchedule(TypedDict, total=False):
    credit_type_id: str
    """Defaults to USD (cents) if not passed."""

    recurring_schedule: CommitInvoiceScheduleRecurringSchedule
    """Enter the unit price and quantity for the charge or instead only send the
    amount.

    If amount is sent, the unit price is assumed to be the amount and quantity is
    inferred to be 1.
    """

    schedule_items: Iterable[CommitInvoiceScheduleScheduleItem]
    """Either provide amount or provide both unit_price and quantity."""


class CommitPaymentGateConfigPrecalculatedTaxConfig(TypedDict, total=False):
    tax_amount: Required[float]
    """Amount of tax to be applied.

    This should be in the same currency and denomination as the commit's invoice
    schedule
    """

    tax_name: str
    """Name of the tax to be applied.

    This may be used in an invoice line item description.
    """


class CommitPaymentGateConfigStripeConfig(TypedDict, total=False):
    payment_type: Required[Literal["INVOICE", "PAYMENT_INTENT"]]
    """If left blank, will default to INVOICE"""

    invoice_metadata: Dict[str, str]
    """Metadata to be added to the Stripe invoice.

    Only applicable if using INVOICE as your payment type.
    """


class CommitPaymentGateConfig(TypedDict, total=False):
    payment_gate_type: Required[Literal["NONE", "STRIPE", "EXTERNAL"]]
    """Gate access to the commit balance based on successful collection of payment.

    Select STRIPE for Metronome to facilitate payment via Stripe. Select EXTERNAL to
    facilitate payment using your own payment integration. Select NONE if you do not
    wish to payment gate the commit balance.
    """

    precalculated_tax_config: CommitPaymentGateConfigPrecalculatedTaxConfig
    """Only applicable if using PRECALCULATED as your tax type."""

    stripe_config: CommitPaymentGateConfigStripeConfig
    """Only applicable if using STRIPE as your payment gate type."""

    tax_type: Literal["NONE", "STRIPE", "ANROK", "PRECALCULATED"]
    """Stripe tax is only supported for Stripe payment gateway.

    Select NONE if you do not wish Metronome to calculate tax on your behalf.
    Leaving this field blank will default to NONE.
    """


class CommitSpecifier(TypedDict, total=False):
    presentation_group_values: Dict[str, str]

    pricing_group_values: Dict[str, str]

    product_id: str
    """
    If provided, the specifier will only apply to the product with the specified ID.
    """

    product_tags: List[str]
    """
    If provided, the specifier will only apply to products with all the specified
    tags.
    """


class Commit(TypedDict, total=False):
    product_id: Required[str]

    type: Required[Literal["PREPAID", "POSTPAID"]]

    access_schedule: CommitAccessSchedule
    """Required: Schedule for distributing the commit to the customer.

    For "POSTPAID" commits only one schedule item is allowed and amount must match
    invoice_schedule total.
    """

    amount: float
    """(DEPRECATED) Use access_schedule and invoice_schedule instead."""

    applicable_product_ids: List[str]
    """Which products the commit applies to.

    If applicable_product_ids, applicable_product_tags or specifiers are not
    provided, the commit applies to all products.
    """

    applicable_product_tags: List[str]
    """Which tags the commit applies to.

    If applicable_product_ids, applicable_product_tags or specifiers are not
    provided, the commit applies to all products.
    """

    custom_fields: Dict[str, str]

    description: str
    """Used only in UI/API. It is not exposed to end customers."""

    hierarchy_configuration: CommitHierarchyConfiguration
    """Optional configuration for commit hierarchy access control"""

    invoice_schedule: CommitInvoiceSchedule
    """
    Required for "POSTPAID" commits: the true up invoice will be generated at this
    time and only one schedule item is allowed; the total must match access_schedule
    amount. Optional for "PREPAID" commits: if not provided, this will be a
    "complimentary" commit with no invoice.
    """

    name: str
    """displayed on invoices"""

    netsuite_sales_order_id: str
    """This field's availability is dependent on your client's configuration."""

    payment_gate_config: CommitPaymentGateConfig
    """optionally payment gate this commit"""

    priority: float
    """
    If multiple commits are applicable, the one with the lower priority will apply
    first.
    """

    rate_type: Literal["COMMIT_RATE", "LIST_RATE"]

    rollover_fraction: float
    """Fraction of unused segments that will be rolled over. Must be between 0 and 1."""

    specifiers: Iterable[CommitSpecifier]
    """
    List of filters that determine what kind of customer usage draws down a commit
    or credit. A customer's usage needs to meet the condition of at least one of the
    specifiers to contribute to a commit's or credit's drawdown. This field cannot
    be used together with `applicable_product_ids` or `applicable_product_tags`.
    """

    temporary_id: str
    """
    A temporary ID for the commit that can be used to reference the commit for
    commit specific overrides.
    """


class CreditAccessScheduleScheduleItem(TypedDict, total=False):
    amount: Required[float]

    ending_before: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]
    """RFC 3339 timestamp (exclusive)"""

    starting_at: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]
    """RFC 3339 timestamp (inclusive)"""


class CreditAccessSchedule(TypedDict, total=False):
    schedule_items: Required[Iterable[CreditAccessScheduleScheduleItem]]

    credit_type_id: str
    """Defaults to USD (cents) if not passed"""


class CreditHierarchyConfigurationChildAccessCommitHierarchyChildAccessAll(TypedDict, total=False):
    type: Required[Literal["ALL"]]


class CreditHierarchyConfigurationChildAccessCommitHierarchyChildAccessNone(TypedDict, total=False):
    type: Required[Literal["NONE"]]


class CreditHierarchyConfigurationChildAccessCommitHierarchyChildAccessContractIDs(TypedDict, total=False):
    contract_ids: Required[List[str]]

    type: Required[Literal["CONTRACT_IDS"]]


CreditHierarchyConfigurationChildAccess: TypeAlias = Union[
    CreditHierarchyConfigurationChildAccessCommitHierarchyChildAccessAll,
    CreditHierarchyConfigurationChildAccessCommitHierarchyChildAccessNone,
    CreditHierarchyConfigurationChildAccessCommitHierarchyChildAccessContractIDs,
]


class CreditHierarchyConfiguration(TypedDict, total=False):
    child_access: Required[CreditHierarchyConfigurationChildAccess]


class CreditSpecifier(TypedDict, total=False):
    presentation_group_values: Dict[str, str]

    pricing_group_values: Dict[str, str]

    product_id: str
    """
    If provided, the specifier will only apply to the product with the specified ID.
    """

    product_tags: List[str]
    """
    If provided, the specifier will only apply to products with all the specified
    tags.
    """


class Credit(TypedDict, total=False):
    access_schedule: Required[CreditAccessSchedule]
    """Schedule for distributing the credit to the customer."""

    product_id: Required[str]

    applicable_product_ids: List[str]
    """Which products the credit applies to.

    If both applicable_product_ids and applicable_product_tags are not provided, the
    credit applies to all products.
    """

    applicable_product_tags: List[str]
    """Which tags the credit applies to.

    If both applicable_product_ids and applicable_product_tags are not provided, the
    credit applies to all products.
    """

    custom_fields: Dict[str, str]

    description: str
    """Used only in UI/API. It is not exposed to end customers."""

    hierarchy_configuration: CreditHierarchyConfiguration
    """Optional configuration for credit hierarchy access control"""

    name: str
    """displayed on invoices"""

    netsuite_sales_order_id: str
    """This field's availability is dependent on your client's configuration."""

    priority: float
    """
    If multiple credits are applicable, the one with the lower priority will apply
    first.
    """

    rate_type: Literal["COMMIT_RATE", "LIST_RATE"]

    specifiers: Iterable[CreditSpecifier]
    """
    List of filters that determine what kind of customer usage draws down a commit
    or credit. A customer's usage needs to meet the condition of at least one of the
    specifiers to contribute to a commit's or credit's drawdown. This field cannot
    be used together with `applicable_product_ids` or `applicable_product_tags`.
    """


class DiscountScheduleRecurringSchedule(TypedDict, total=False):
    amount_distribution: Required[Literal["DIVIDED", "DIVIDED_ROUNDED", "EACH"]]

    ending_before: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]
    """RFC 3339 timestamp (exclusive)."""

    frequency: Required[Literal["MONTHLY", "QUARTERLY", "SEMI_ANNUAL", "ANNUAL"]]

    starting_at: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]
    """RFC 3339 timestamp (inclusive)."""

    amount: float
    """Amount for the charge.

    Can be provided instead of unit_price and quantity. If amount is sent, the
    unit_price is assumed to be the amount and quantity is inferred to be 1.
    """

    quantity: float
    """Quantity for the charge.

    Will be multiplied by unit_price to determine the amount and must be specified
    with unit_price. If specified amount cannot be provided.
    """

    unit_price: float
    """Unit price for the charge.

    Will be multiplied by quantity to determine the amount and must be specified
    with quantity. If specified amount cannot be provided.
    """


class DiscountScheduleScheduleItem(TypedDict, total=False):
    timestamp: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]
    """timestamp of the scheduled event"""

    amount: float
    """Amount for the charge.

    Can be provided instead of unit_price and quantity. If amount is sent, the
    unit_price is assumed to be the amount and quantity is inferred to be 1.
    """

    quantity: float
    """Quantity for the charge.

    Will be multiplied by unit_price to determine the amount and must be specified
    with unit_price. If specified amount cannot be provided.
    """

    unit_price: float
    """Unit price for the charge.

    Will be multiplied by quantity to determine the amount and must be specified
    with quantity. If specified amount cannot be provided.
    """


class DiscountSchedule(TypedDict, total=False):
    credit_type_id: str
    """Defaults to USD (cents) if not passed."""

    recurring_schedule: DiscountScheduleRecurringSchedule
    """Enter the unit price and quantity for the charge or instead only send the
    amount.

    If amount is sent, the unit price is assumed to be the amount and quantity is
    inferred to be 1.
    """

    schedule_items: Iterable[DiscountScheduleScheduleItem]
    """Either provide amount or provide both unit_price and quantity."""


class Discount(TypedDict, total=False):
    product_id: Required[str]

    schedule: Required[DiscountSchedule]
    """Must provide either schedule_items or recurring_schedule."""

    custom_fields: Dict[str, str]

    name: str
    """displayed on invoices"""

    netsuite_sales_order_id: str
    """This field's availability is dependent on your client's configuration."""


class OverrideOverrideSpecifier(TypedDict, total=False):
    billing_frequency: Literal["MONTHLY", "QUARTERLY", "ANNUAL", "WEEKLY"]

    commit_ids: List[str]
    """Can only be used for commit specific overrides.

    Must be used in conjunction with one of product_id, product_tags,
    pricing_group_values, or presentation_group_values. If provided, the override
    will only apply to the specified commits. If not provided, the override will
    apply to all commits.
    """

    presentation_group_values: Dict[str, str]
    """A map of group names to values.

    The override will only apply to line items with the specified presentation group
    values.
    """

    pricing_group_values: Dict[str, str]
    """A map of pricing group names to values.

    The override will only apply to products with the specified pricing group
    values.
    """

    product_id: str
    """If provided, the override will only apply to the product with the specified ID."""

    product_tags: List[str]
    """
    If provided, the override will only apply to products with all the specified
    tags.
    """

    recurring_commit_ids: List[str]
    """Can only be used for commit specific overrides.

    Must be used in conjunction with one of product_id, product_tags,
    pricing_group_values, or presentation_group_values. If provided, the override
    will only apply to commits created by the specified recurring commit ids.
    """

    recurring_credit_ids: List[str]
    """Can only be used for commit specific overrides.

    Must be used in conjunction with one of product_id, product_tags,
    pricing_group_values, or presentation_group_values. If provided, the override
    will only apply to credits created by the specified recurring credit ids.
    """


class OverrideOverwriteRate(TypedDict, total=False):
    rate_type: Required[Literal["FLAT", "PERCENTAGE", "SUBSCRIPTION", "TIERED", "CUSTOM"]]

    credit_type_id: str

    custom_rate: Dict[str, object]
    """Only set for CUSTOM rate_type.

    This field is interpreted by custom rate processors.
    """

    is_prorated: bool
    """Default proration configuration.

    Only valid for SUBSCRIPTION rate_type. Must be set to true.
    """

    price: float
    """Default price.

    For FLAT rate_type, this must be >=0. For PERCENTAGE rate_type, this is a
    decimal fraction, e.g. use 0.1 for 10%; this must be >=0 and <=1.
    """

    quantity: float
    """Default quantity. For SUBSCRIPTION rate_type, this must be >=0."""

    tiers: Iterable[Tier]
    """Only set for TIERED rate_type."""


class OverrideTier(TypedDict, total=False):
    multiplier: Required[float]

    size: float


class Override(TypedDict, total=False):
    starting_at: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]
    """RFC 3339 timestamp indicating when the override will start applying (inclusive)"""

    applicable_product_tags: List[str]
    """tags identifying products whose rates are being overridden.

    Cannot be used in conjunction with override_specifiers.
    """

    ending_before: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """RFC 3339 timestamp indicating when the override will stop applying (exclusive)"""

    entitled: bool

    is_commit_specific: bool
    """Indicates whether the override should only apply to commits.

    Defaults to `false`. If `true`, you can specify relevant commits in
    `override_specifiers` by passing `commit_ids`. if you do not specify
    `commit_ids`, then the override will apply when consuming any prepaid or
    postpaid commit.
    """

    multiplier: float
    """Required for MULTIPLIER type. Must be >=0."""

    override_specifiers: Iterable[OverrideOverrideSpecifier]
    """Cannot be used in conjunction with product_id or applicable_product_tags.

    If provided, the override will apply to all products with the specified
    specifiers.
    """

    overwrite_rate: OverrideOverwriteRate
    """Required for OVERWRITE type."""

    priority: float
    """Required for EXPLICIT multiplier prioritization scheme and all TIERED overrides.

    Under EXPLICIT prioritization, overwrites are prioritized first, and then tiered
    and multiplier overrides are prioritized by their priority value (lowest first).
    Must be > 0.
    """

    product_id: str
    """ID of the product whose rate is being overridden.

    Cannot be used in conjunction with override_specifiers.
    """

    target: Literal["COMMIT_RATE", "LIST_RATE"]
    """Indicates whether the override applies to commit rates or list rates.

    Can only be used for overrides that have `is_commit_specific` set to `true`.
    Defaults to `"LIST_RATE"`.
    """

    tiers: Iterable[OverrideTier]
    """Required for TIERED type. Must have at least one tier."""

    type: Literal["OVERWRITE", "MULTIPLIER", "TIERED"]
    """Overwrites are prioritized over multipliers and tiered overrides."""


class ProfessionalService(TypedDict, total=False):
    max_amount: Required[float]
    """Maximum amount for the term."""

    product_id: Required[str]

    quantity: Required[float]
    """Quantity for the charge.

    Will be multiplied by unit_price to determine the amount.
    """

    unit_price: Required[float]
    """Unit price for the charge.

    Will be multiplied by quantity to determine the amount and must be specified.
    """

    custom_fields: Dict[str, str]

    description: str

    netsuite_sales_order_id: str
    """This field's availability is dependent on your client's configuration."""


class ResellerRoyaltyAwsOptions(TypedDict, total=False):
    aws_account_number: str

    aws_offer_id: str

    aws_payer_reference_id: str


class ResellerRoyaltyGcpOptions(TypedDict, total=False):
    gcp_account_id: str

    gcp_offer_id: str


class ResellerRoyalty(TypedDict, total=False):
    reseller_type: Required[Literal["AWS", "AWS_PRO_SERVICE", "GCP", "GCP_PRO_SERVICE"]]

    applicable_product_ids: List[str]
    """Must provide at least one of applicable_product_ids or applicable_product_tags."""

    applicable_product_tags: List[str]
    """Must provide at least one of applicable_product_ids or applicable_product_tags."""

    aws_options: ResellerRoyaltyAwsOptions

    ending_before: Annotated[Union[str, datetime, None], PropertyInfo(format="iso8601")]
    """Use null to indicate that the existing end timestamp should be removed."""

    fraction: float

    gcp_options: ResellerRoyaltyGcpOptions

    netsuite_reseller_id: str

    reseller_contract_value: float

    starting_at: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]


class ScheduledChargeScheduleRecurringSchedule(TypedDict, total=False):
    amount_distribution: Required[Literal["DIVIDED", "DIVIDED_ROUNDED", "EACH"]]

    ending_before: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]
    """RFC 3339 timestamp (exclusive)."""

    frequency: Required[Literal["MONTHLY", "QUARTERLY", "SEMI_ANNUAL", "ANNUAL"]]

    starting_at: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]
    """RFC 3339 timestamp (inclusive)."""

    amount: float
    """Amount for the charge.

    Can be provided instead of unit_price and quantity. If amount is sent, the
    unit_price is assumed to be the amount and quantity is inferred to be 1.
    """

    quantity: float
    """Quantity for the charge.

    Will be multiplied by unit_price to determine the amount and must be specified
    with unit_price. If specified amount cannot be provided.
    """

    unit_price: float
    """Unit price for the charge.

    Will be multiplied by quantity to determine the amount and must be specified
    with quantity. If specified amount cannot be provided.
    """


class ScheduledChargeScheduleScheduleItem(TypedDict, total=False):
    timestamp: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]
    """timestamp of the scheduled event"""

    amount: float
    """Amount for the charge.

    Can be provided instead of unit_price and quantity. If amount is sent, the
    unit_price is assumed to be the amount and quantity is inferred to be 1.
    """

    quantity: float
    """Quantity for the charge.

    Will be multiplied by unit_price to determine the amount and must be specified
    with unit_price. If specified amount cannot be provided.
    """

    unit_price: float
    """Unit price for the charge.

    Will be multiplied by quantity to determine the amount and must be specified
    with quantity. If specified amount cannot be provided.
    """


class ScheduledChargeSchedule(TypedDict, total=False):
    credit_type_id: str
    """Defaults to USD (cents) if not passed."""

    recurring_schedule: ScheduledChargeScheduleRecurringSchedule
    """Enter the unit price and quantity for the charge or instead only send the
    amount.

    If amount is sent, the unit price is assumed to be the amount and quantity is
    inferred to be 1.
    """

    schedule_items: Iterable[ScheduledChargeScheduleScheduleItem]
    """Either provide amount or provide both unit_price and quantity."""


class ScheduledCharge(TypedDict, total=False):
    product_id: Required[str]

    schedule: Required[ScheduledChargeSchedule]
    """Must provide either schedule_items or recurring_schedule."""

    name: str
    """displayed on invoices"""

    netsuite_sales_order_id: str
    """This field's availability is dependent on your client's configuration."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict
from typing_extensions import Literal, Required, TypedDict

__all__ = ["CustomFieldSetValuesParams"]


class CustomFieldSetValuesParams(TypedDict, total=False):
    custom_fields: Required[Dict[str, str]]

    entity: Required[
        Literal[
            "alert",
            "billable_metric",
            "charge",
            "commit",
            "contract_credit",
            "contract_product",
            "contract",
            "credit_grant",
            "customer_plan",
            "customer",
            "discount",
            "invoice",
            "plan",
            "professional_service",
            "product",
            "rate_card",
            "scheduled_charge",
            "subscription",
        ]
    ]

    entity_id: Required[str]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, Union, Mapping
from typing_extensions import Self, override

import httpx

from . import _exceptions
from ._qs import Querystring
from ._types import (
    NOT_GIVEN,
    Omit,
    Timeout,
    NotGiven,
    Transport,
    ProxiesTypes,
    RequestOptions,
)
from ._utils import is_given, get_async_library
from ._version import __version__
from ._streaming import Stream as Stream, AsyncStream as AsyncStream
from ._exceptions import APIStatusError, MetronomeError
from ._base_client import (
    DEFAULT_MAX_RETRIES,
    SyncAPIClient,
    AsyncAPIClient,
)
from .resources.v1 import v1
from .resources.v2 import v2

__all__ = [
    "Timeout",
    "Transport",
    "ProxiesTypes",
    "RequestOptions",
    "Metronome",
    "AsyncMetronome",
    "Client",
    "AsyncClient",
]


class Metronome(SyncAPIClient):
    v2: v2.V2Resource
    v1: v1.V1Resource
    with_raw_response: MetronomeWithRawResponse
    with_streaming_response: MetronomeWithStreamedResponse

    # client options
    bearer_token: str
    webhook_secret: str | None

    def __init__(
        self,
        *,
        bearer_token: str | None = None,
        webhook_secret: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        max_retries: int = DEFAULT_MAX_RETRIES,
        default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        # Configure a custom httpx client.
        # We provide a `DefaultHttpxClient` class that you can pass to retain the default values we use for `limits`, `timeout` & `follow_redirects`.
        # See the [httpx documentation](https://www.python-httpx.org/api/#client) for more details.
        http_client: httpx.Client | None = None,
        # Enable or disable schema validation for data returned by the API.
        # When enabled an error APIResponseValidationError is raised
        # if the API responds with invalid data for the expected schema.
        #
        # This parameter may be removed or changed in the future.
        # If you rely on this feature, please open a GitHub issue
        # outlining your use-case to help us decide if it should be
        # part of our public interface in the future.
        _strict_response_validation: bool = False,
    ) -> None:
        """Construct a new synchronous Metronome client instance.

        This automatically infers the following arguments from their corresponding environment variables if they are not provided:
        - `bearer_token` from `METRONOME_BEARER_TOKEN`
        - `webhook_secret` from `METRONOME_WEBHOOK_SECRET`
        """
        if bearer_token is None:
            bearer_token = os.environ.get("METRONOME_BEARER_TOKEN")
        if bearer_token is None:
            raise MetronomeError(
                "The bearer_token client option must be set either by passing bearer_token to the client or by setting the METRONOME_BEARER_TOKEN environment variable"
            )
        self.bearer_token = bearer_token

        if webhook_secret is None:
            webhook_secret = os.environ.get("METRONOME_WEBHOOK_SECRET")
        self.webhook_secret = webhook_secret

        if base_url is None:
            base_url = os.environ.get("METRONOME_BASE_URL")
        if base_url is None:
            base_url = f"https://api.metronome.com"

        super().__init__(
            version=__version__,
            base_url=base_url,
            max_retries=max_retries,
            timeout=timeout,
            http_client=http_client,
            custom_headers=default_headers,
            custom_query=default_query,
            _strict_response_validation=_strict_response_validation,
        )

        self.v2 = v2.V2Resource(self)
        self.v1 = v1.V1Resource(self)
        self.with_raw_response = MetronomeWithRawResponse(self)
        self.with_streaming_response = MetronomeWithStreamedResponse(self)

    @property
    @override
    def qs(self) -> Querystring:
        return Querystring(array_format="comma")

    @property
    @override
    def auth_headers(self) -> dict[str, str]:
        bearer_token = self.bearer_token
        return {"Authorization": f"Bearer {bearer_token}"}

    @property
    @override
    def default_headers(self) -> dict[str, str | Omit]:
        return {
            **super().default_headers,
            "X-Stainless-Async": "false",
            **self._custom_headers,
        }

    def copy(
        self,
        *,
        bearer_token: str | None = None,
        webhook_secret: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = NOT_GIVEN,
        http_client: httpx.Client | None = None,
        max_retries: int | NotGiven = NOT_GIVEN,
        default_headers: Mapping[str, str] | None = None,
        set_default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        set_default_query: Mapping[str, object] | None = None,
        _extra_kwargs: Mapping[str, Any] = {},
    ) -> Self:
        """
        Create a new client instance re-using the same options given to the current client with optional overriding.
        """
        if default_headers is not None and set_default_headers is not None:
            raise ValueError("The `default_headers` and `set_default_headers` arguments are mutually exclusive")

        if default_query is not None and set_default_query is not None:
            raise ValueError("The `default_query` and `set_default_query` arguments are mutually exclusive")

        headers = self._custom_headers
        if default_headers is not None:
            headers = {**headers, **default_headers}
        elif set_default_headers is not None:
            headers = set_default_headers

        params = self._custom_query
        if default_query is not None:
            params = {**params, **default_query}
        elif set_default_query is not None:
            params = set_default_query

        http_client = http_client or self._client
        return self.__class__(
            bearer_token=bearer_token or self.bearer_token,
            webhook_secret=webhook_secret or self.webhook_secret,
            base_url=base_url or self.base_url,
            timeout=self.timeout if isinstance(timeout, NotGiven) else timeout,
            http_client=http_client,
            max_retries=max_retries if is_given(max_retries) else self.max_retries,
            default_headers=headers,
            default_query=params,
            **_extra_kwargs,
        )

    # Alias for `copy` for nicer inline usage, e.g.
    # client.with_options(timeout=10).foo.create(...)
    with_options = copy

    @override
    def _make_status_error(
        self,
        err_msg: str,
        *,
        body: object,
        response: httpx.Response,
    ) -> APIStatusError:
        if response.status_code == 400:
            return _exceptions.BadRequestError(err_msg, response=response, body=body)

        if response.status_code == 401:
            return _exceptions.AuthenticationError(err_msg, response=response, body=body)

        if response.status_code == 403:
            return _exceptions.PermissionDeniedError(err_msg, response=response, body=body)

        if response.status_code == 404:
            return _exceptions.NotFoundError(err_msg, response=response, body=body)

        if response.status_code == 409:
            return _exceptions.ConflictError(err_msg, response=response, body=body)

        if response.status_code == 422:
            return _exceptions.UnprocessableEntityError(err_msg, response=response, body=body)

        if response.status_code == 429:
            return _exceptions.RateLimitError(err_msg, response=response, body=body)

        if response.status_code >= 500:
            return _exceptions.InternalServerError(err_msg, response=response, body=body)
        return APIStatusError(err_msg, response=response, body=body)


class AsyncMetronome(AsyncAPIClient):
    v2: v2.AsyncV2Resource
    v1: v1.AsyncV1Resource
    with_raw_response: AsyncMetronomeWithRawResponse
    with_streaming_response: AsyncMetronomeWithStreamedResponse

    # client options
    bearer_token: str
    webhook_secret: str | None

    def __init__(
        self,
        *,
        bearer_token: str | None = None,
        webhook_secret: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        max_retries: int = DEFAULT_MAX_RETRIES,
        default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        # Configure a custom httpx client.
        # We provide a `DefaultAsyncHttpxClient` class that you can pass to retain the default values we use for `limits`, `timeout` & `follow_redirects`.
        # See the [httpx documentation](https://www.python-httpx.org/api/#asyncclient) for more details.
        http_client: httpx.AsyncClient | None = None,
        # Enable or disable schema validation for data returned by the API.
        # When enabled an error APIResponseValidationError is raised
        # if the API responds with invalid data for the expected schema.
        #
        # This parameter may be removed or changed in the future.
        # If you rely on this feature, please open a GitHub issue
        # outlining your use-case to help us decide if it should be
        # part of our public interface in the future.
        _strict_response_validation: bool = False,
    ) -> None:
        """Construct a new async AsyncMetronome client instance.

        This automatically infers the following arguments from their corresponding environment variables if they are not provided:
        - `bearer_token` from `METRONOME_BEARER_TOKEN`
        - `webhook_secret` from `METRONOME_WEBHOOK_SECRET`
        """
        if bearer_token is None:
            bearer_token = os.environ.get("METRONOME_BEARER_TOKEN")
        if bearer_token is None:
            raise MetronomeError(
                "The bearer_token client option must be set either by passing bearer_token to the client or by setting the METRONOME_BEARER_TOKEN environment variable"
            )
        self.bearer_token = bearer_token

        if webhook_secret is None:
            webhook_secret = os.environ.get("METRONOME_WEBHOOK_SECRET")
        self.webhook_secret = webhook_secret

        if base_url is None:
            base_url = os.environ.get("METRONOME_BASE_URL")
        if base_url is None:
            base_url = f"https://api.metronome.com"

        super().__init__(
            version=__version__,
            base_url=base_url,
            max_retries=max_retries,
            timeout=timeout,
            http_client=http_client,
            custom_headers=default_headers,
            custom_query=default_query,
            _strict_response_validation=_strict_response_validation,
        )

        self.v2 = v2.AsyncV2Resource(self)
        self.v1 = v1.AsyncV1Resource(self)
        self.with_raw_response = AsyncMetronomeWithRawResponse(self)
        self.with_streaming_response = AsyncMetronomeWithStreamedResponse(self)

    @property
    @override
    def qs(self) -> Querystring:
        return Querystring(array_format="comma")

    @property
    @override
    def auth_headers(self) -> dict[str, str]:
        bearer_token = self.bearer_token
        return {"Authorization": f"Bearer {bearer_token}"}

    @property
    @override
    def default_headers(self) -> dict[str, str | Omit]:
        return {
            **super().default_headers,
            "X-Stainless-Async": f"async:{get_async_library()}",
            **self._custom_headers,
        }

    def copy(
        self,
        *,
        bearer_token: str | None = None,
        webhook_secret: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = NOT_GIVEN,
        http_client: httpx.AsyncClient | None = None,
        max_retries: int | NotGiven = NOT_GIVEN,
        default_headers: Mapping[str, str] | None = None,
        set_default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        set_default_query: Mapping[str, object] | None = None,
        _extra_kwargs: Mapping[str, Any] = {},
    ) -> Self:
        """
        Create a new client instance re-using the same options given to the current client with optional overriding.
        """
        if default_headers is not None and set_default_headers is not None:
            raise ValueError("The `default_headers` and `set_default_headers` arguments are mutually exclusive")

        if default_query is not None and set_default_query is not None:
            raise ValueError("The `default_query` and `set_default_query` arguments are mutually exclusive")

        headers = self._custom_headers
        if default_headers is not None:
            headers = {**headers, **default_headers}
        elif set_default_headers is not None:
            headers = set_default_headers

        params = self._custom_query
        if default_query is not None:
            params = {**params, **default_query}
        elif set_default_query is not None:
            params = set_default_query

        http_client = http_client or self._client
        return self.__class__(
            bearer_token=bearer_token or self.bearer_token,
            webhook_secret=webhook_secret or self.webhook_secret,
            base_url=base_url or self.base_url,
            timeout=self.timeout if isinstance(timeout, NotGiven) else timeout,
            http_client=http_client,
            max_retries=max_retries if is_given(max_retries) else self.max_retries,
            default_headers=headers,
            default_query=params,
            **_extra_kwargs,
        )

    # Alias for `copy` for nicer inline usage, e.g.
    # client.with_options(timeout=10).foo.create(...)
    with_options = copy

    @override
    def _make_status_error(
        self,
        err_msg: str,
        *,
        body: object,
        response: httpx.Response,
    ) -> APIStatusError:
        if response.status_code == 400:
            return _exceptions.BadRequestError(err_msg, response=response, body=body)

        if response.status_code == 401:
            return _exceptions.AuthenticationError(err_msg, response=response, body=body)

        if response.status_code == 403:
            return _exceptions.PermissionDeniedError(err_msg, response=response, body=body)

        if response.status_code == 404:
            return _exceptions.NotFoundError(err_msg, response=response, body=body)

        if response.status_code == 409:
            return _exceptions.ConflictError(err_msg, response=response, body=body)

        if response.status_code == 422:
            return _exceptions.UnprocessableEntityError(err_msg, response=response, body=body)

        if response.status_code == 429:
            return _exceptions.RateLimitError(err_msg, response=response, body=body)

        if response.status_code >= 500:
            return _exceptions.InternalServerError(err_msg, response=response, body=body)
        return APIStatusError(err_msg, response=response, body=body)


class MetronomeWithRawResponse:
    def __init__(self, client: Metronome) -> None:
        self.v2 = v2.V2ResourceWithRawResponse(client.v2)
        self.v1 = v1.V1ResourceWithRawResponse(client.v1)


class AsyncMetronomeWithRawResponse:
    def __init__(self, client: AsyncMetronome) -> None:
        self.v2 = v2.AsyncV2ResourceWithRawResponse(client.v2)
        self.v1 = v1.AsyncV1ResourceWithRawResponse(client.v1)


class MetronomeWithStreamedResponse:
    def __init__(self, client: Metronome) -> None:
        self.v2 = v2.V2ResourceWithStreamingResponse(client.v2)
        self.v1 = v1.V1ResourceWithStreamingResponse(client.v1)


class AsyncMetronomeWithStreamedResponse:
    def __init__(self, client: AsyncMetronome) -> None:
        self.v2 = v2.AsyncV2ResourceWithStreamingResponse(client.v2)
        self.v1 = v1.AsyncV1ResourceWithStreamingResponse(client.v1)


Client = Metronome

AsyncClient = AsyncMetronome



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime

from ..._models import BaseModel
from .credit_ledger_entry import CreditLedgerEntry
from ..shared.credit_type_data import CreditTypeData

__all__ = [
    "CreditGrantListEntriesResponse",
    "Data",
    "DataLedger",
    "DataLedgerEndingBalance",
    "DataLedgerStartingBalance",
]


class DataLedgerEndingBalance(BaseModel):
    effective_at: datetime
    """
    the ending_before request parameter (if supplied) or the current billing
    period's end date
    """

    excluding_pending: float
    """
    the ending balance, including the balance of all grants that have not expired
    before the effective_at date and deductions that happened before the
    effective_at date
    """

    including_pending: float
    """
    the excluding_pending balance plus any pending invoice deductions and
    expirations that will happen by the effective_at date
    """


class DataLedgerStartingBalance(BaseModel):
    effective_at: datetime
    """
    the starting_on request parameter (if supplied) or the first credit grant's
    effective_at date
    """

    excluding_pending: float
    """
    the starting balance, including all posted grants, deductions, and expirations
    that happened at or before the effective_at timestamp
    """

    including_pending: float
    """
    the excluding_pending balance plus any pending activity that has not been posted
    at the time of the query
    """


class DataLedger(BaseModel):
    credit_type: CreditTypeData

    ending_balance: DataLedgerEndingBalance
    """the effective balances at the end of the specified time window"""

    entries: List[CreditLedgerEntry]

    pending_entries: List[CreditLedgerEntry]

    starting_balance: DataLedgerStartingBalance


class Data(BaseModel):
    customer_id: str

    ledgers: List[DataLedger]


class CreditGrantListEntriesResponse(BaseModel):
    data: List[Data]

    next_page: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .invoice import Invoice
from ...._models import BaseModel

__all__ = ["InvoiceRetrieveResponse"]


class InvoiceRetrieveResponse(BaseModel):
    data: Invoice
